//
//  kjots
//
//  Copyright (C) 1997 Christoph Neerfeld
//  Copyright (C) 2002, 2003 Aaron J. Seigo
//  Copyright (C) 2003 Stanislav Kljuhhin
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#include <iostream>
#include <qbuttongroup.h>
#include <qcursor.h>
#include <qobjectlist.h>
#include <qdatetime.h>
#include <qlabel.h>
#include <qtimer.h>

#include <kaccel.h>
#include <kaction.h>
#include <kstdaccel.h>
#include <kstdaction.h>
#include <kapplication.h>
#include <kedittoolbar.h>
#include <kfiledialog.h>
#include <kdebug.h>
#include <kkeydialog.h>
#include <kinputdialog.h>
#include <klocale.h>
#include <kmenubar.h>
#include <kmessagebox.h>
#include <kopenwith.h>
#include <kpopupmenu.h>
#include <krun.h>
#include <kstandarddirs.h>
#include <kstringhandler.h>
#include <kglobalsettings.h>

#include <kurl.h>
#include <keditcl.h>
#include <kprinter.h>
#include <kglobal.h>
#include <kiconloader.h>

#include "KJotsMain.h"
#include "optiondialog.h"
#include "kjotsentry.h"
#include "kjotsedit.h"

#define KJOTSVERSION "0.6"

//
// KMyListView
//

extern KIconLoader* global_pix_loader;

//----------------------------------------------------------------------
// KJOTSMAIN
//----------------------------------------------------------------------
KJotsMain::KJotsMain(const char* name)
    : KMainWindow(0,name),
      mOptionDialog(0),
      currentEntry(0)
{
    //
    // Main widget
    //
    QWidget* f_main = new QFrame( this, "Frame_0" );
    setCentralWidget(f_main);

    QSplitter* splitter = new QSplitter(f_main);
    splitter->setOpaqueResize();

    // the subject list
    subjectList = new KListView(splitter, "subjectList");
    subjectList->setRootIsDecorated(true);
    subjectList->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding));
    subjectList->setMinimumWidth(subjectList->fontMetrics().maxWidth() * 10 + 5);
    subjectList->addColumn(i18n("Pages"));
    subjectList->setFullWidth(true);
    subjectList->setHScrollBarMode(QScrollView::AlwaysOff);
    subjectList->setDefaultRenameAction(QListView::Accept);
    subjectList->setSorting(-1);
    subjectList->setAlternateBackground(QColor());
    //subjectList->header()->hide();
    subjectList->setDragEnabled(true);
    subjectList->setAcceptDrops(true);
    subjectList->setItemsMovable(true);
    subjectList->setItemsRenameable(true);
    subjectList->setRenameable(0,true);
    connect(subjectList, SIGNAL(moved(QListViewItem*, QListViewItem*, QListViewItem*)),
            this, SLOT(slotItemMoved(QListViewItem*, QListViewItem*, QListViewItem*)));

    me_text = new KJotsEdit(splitter, "me_text");
    me_text->setFocusPolicy(QWidget::StrongFocus);
    me_text->setEnabled(false);
    connect(me_text, SIGNAL(findSuccessful()), this, SLOT(slotFindSuccessful()));

    QVBoxLayout *bookGrid = new QVBoxLayout(f_main, KDialog::marginHint(), KDialog::spacingHint());
    bookGrid->addWidget(splitter, 0, 0);
    splitter->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding, 2, 1));

    updateConfiguration();

    // TODO: CTRL+Key_M was move focus, should now be rename page? erk
    // create menu
    KAction *action = KStdAction::openNew(this, SLOT(newEntry()), actionCollection());
    action->setText(i18n("&New Page..."));

    action = new KAction(i18n("New &Book"), "contents", CTRL+SHIFT+Key_N, this,
            SLOT(createFolder()), actionCollection(), "new_book");
    action = new KAction(i18n("Save Book to ASCII File..."), QString::null, 0, this,
            SLOT(writeBook()), actionCollection(), "save_book_to_ascii");
    action = new KAction(i18n("Save Page to ASCII File..."), QString::null, 0, this,
            SLOT(writePage()), actionCollection(), "save_page_to_ascii");

    action = new KAction(i18n("Delete Boo&k"), "editdelete", 0, this,
            SLOT(deleteFolder()), actionCollection(), "del_folder");

    action = new KAction(i18n("&Delete Page"), "editdelete", 0, this,
            SLOT(deleteEntry()), actionCollection(), "del_page");

    action = KStdAction::print(this, SLOT(slotPrintPage()), actionCollection());
    action = KStdAction::quit(this, SLOT(slotQuit()), actionCollection());

    action = KStdAction::cut(me_text, SLOT(cut()), actionCollection());
    connect(me_text, SIGNAL(copyAvailable(bool)), action, SLOT(setEnabled(bool)));
    action->setEnabled(false);
    action  = KStdAction::copy(me_text, SLOT(copy()), actionCollection());
    connect(me_text, SIGNAL(copyAvailable(bool)), action, SLOT(setEnabled(bool)));
    action->setEnabled(false);
    action = new KAction(i18n("Copy &into Page Title"), "editcopy", CTRL+Key_T, this,
            SLOT(copySelection()), actionCollection(), "copyIntoTitle");
    connect(me_text, SIGNAL(copyAvailable(bool)), action, SLOT(setEnabled(bool)));
    action->setEnabled(false);
    action  = KStdAction::paste(me_text, SLOT(paste()), actionCollection());

    action = KStdAction::find( this, SLOT( slotSearch() ), actionCollection() );
    action = KStdAction::findNext( this, SLOT( slotRepeatSearch() ), actionCollection() );
    action->setEnabled(false);
    action = KStdAction::replace( this, SLOT( slotReplace() ), actionCollection() );

    action = new KAction(i18n("Insert Date"), "date", CTRL+Key_I, this,
            SLOT(insertDate()), actionCollection(), "insert_date");

    action = KStdAction::showToolbar(this, SLOT(slotShowHideToolbars()), actionCollection());
    action = KStdAction::keyBindings(this, SLOT(configureKeys()), actionCollection());
    action = KStdAction::configureToolbars(this, SLOT(configureToolbars()), actionCollection());
    action = KStdAction::preferences(this, SLOT(configure()), actionCollection());
    createGUI();

    //
    // Set startup size.
    //
    KConfig *config = kapp->config();
    config->setGroup("kjots");

    QDir dir(locateLocal("appdata",""));
    QString absPath=dir.absPath()+'/';
    QStringList files=dir.entryList(QDir::Files|QDir::Readable);

    for (QStringList::Iterator it=files.begin();it!=files.end();it++)
    {
        QString decodedFilename=*it;
        new KJotsBook(subjectList,decodedFilename);
    }

    updateConfiguration();
    updateMenu();

    int w = config->readNumEntry("Width", 0);
    int h = config->readNumEntry("Height", 0);
    if (w == 0)
    {
        w = 600;
    }
    if (h == 0)
    {
        h = 400;
    }
    resize(w, h);

    QTimer::singleShot(0, this, SLOT(slotSetupInterface()));

    //connect new slots
    connect(subjectList,SIGNAL(expanded(QListViewItem*)),this,SLOT(slotExpandBook(QListViewItem*)));
    connect(subjectList,SIGNAL(collapsed(QListViewItem*)),this,SLOT(slotCollapseBook(QListViewItem*)));
    connect(subjectList,SIGNAL(selectionChanged(QListViewItem*)),this,SLOT(jumpToEntry(QListViewItem*)));
    connect(subjectList,SIGNAL(contextMenu(KListView*,QListViewItem*,const QPoint&)),this,SLOT(showListviewContextenu(KListView*,QListViewItem*,const QPoint&)));
    connect(subjectList,SIGNAL(itemRenamed(QListViewItem*,const QString&,int)),this,SLOT(slotItemRenamed(QListViewItem*,const QString&,int)));
}

void KJotsMain::slotSetupInterface()
{
    static_cast<KToggleAction*>(actionCollection()->action("options_show_toolbar"))->setChecked(toolBar()->isVisible());

    if (subjectList->childCount()==0)
        createFolder();
}



KJotsMain::~KJotsMain()
{
    delete mOptionDialog;
    saveProperties();
}


void KJotsMain::saveProperties( void )
{
  KConfig* config = kapp->config();
  config->setGroup("kjots");
  config->writeEntry("Width", width());
  config->writeEntry("Height", height());
  toolBar(0)->saveSettings(config, "MainToolbar");
  config->sync();
}

bool KJotsMain::createFolder()
{
    bool ok;

    QString name = KInputDialog::getText( i18n( "New Book" ),
        i18n( "Book name:" ), QString::null, &ok, this );

    if (!ok)
        return false;

    if(subjectList->findItem(name,0))
    {
        KMessageBox::sorry(topLevelWidget(),i18n("A book with this name already exists."),"KJots");
        return false;
    }

    KJotsBook* book=new KJotsBook(subjectList,name);
    KJotsSubject* subj=new KJotsSubject(book,QString::null);
    book->setOpen(true);
    subjectList->setSelected(subj,true);
    return true;
}

void KJotsMain::deleteFolder()
{
    KJotsBook* b=currentBook();

    if (!b)
        return;

    QString msg=i18n("Are you sure you want to delete this book?");
    int result=KMessageBox::warningYesNo(topLevelWidget(),msg,i18n("Delete Book"));

    if (result!=KMessageBox::Yes)
        return;

    b->setDeleteFlag();
    delete b;
    updateMenu();
}

void KJotsMain::newEntry()
{
    KJotsBook* b = currentBook();

    if (b)
    {
        KJotsSubject* s = new KJotsSubject(b,QString::null);
        subjectList->setSelected(s, true);
    }
}

void KJotsMain::deleteEntry()
{
    KJotsSubject* cur=dynamic_cast<KJotsSubject*>(subjectList->currentItem());

    if (!cur)
        return;

    if (KMessageBox::warningYesNo(topLevelWidget(),
                i18n("Are you sure you want to delete this page?"),
                i18n("Delete Page"))!=KMessageBox::Yes)
        return;

    KJotsBook* bk=dynamic_cast<KJotsBook*>(cur->parent());

    KJotsSubject* nxt=dynamic_cast<KJotsSubject*>(cur->nextSibling());

    delete cur;
    currentEntry=0;

    if (!nxt)
        nxt=dynamic_cast<KJotsSubject*>(bk->firstChild());

    if (nxt)
        subjectList->setSelected(nxt, true);
    else
    {
        me_text->clear();
        setCaption(kapp->makeStdCaption(QString::null));
    }
    updateMenu();
}

void KJotsMain::configureToolbars()
{
  saveMainWindowSettings(kapp->config(), "kjots");
  KEditToolbar dlg(actionCollection());
  connect( &dlg, SIGNAL(newToolbarConfig()), SLOT(updateToolbars()) );
  dlg.exec();
}

void KJotsMain::updateToolbars()
{
  createGUI();
  applyMainWindowSettings(kapp->config(), "kjots");
  updateMenu();
}

void KJotsMain::configure()
{
  if( mOptionDialog == 0 )
  {
    mOptionDialog = new ConfigureDialog( topLevelWidget(), 0, false );
    if( mOptionDialog == 0 ) { return; }
    connect( mOptionDialog, SIGNAL(hidden()),this,SLOT(configureHide()));
    connect( mOptionDialog, SIGNAL(valueChanged()),
             this, SLOT(updateConfiguration()) );
  }

  mOptionDialog->show();
}

void KJotsMain::configureHide()
{
  QTimer::singleShot( 0, this, SLOT(configureDestroy()) );
}

void KJotsMain::configureDestroy()
{
  if( mOptionDialog != 0 && mOptionDialog->isVisible() == false )
  {
    delete mOptionDialog; mOptionDialog = 0;
  }
}

void KJotsMain::slotSearch()
{
    me_text->search();
}

void KJotsMain::slotRepeatSearch()
{
    me_text->repeatSearch();
}

void KJotsMain::slotReplace()
{
    me_text->replace();
}

void KJotsMain::slotShowHideToolbars()
{
    if (toolBar()->isVisible())
    {
        toolBar()->hide();
    }
    else
    {
        toolBar()->show();
    }
}

void KJotsMain::updateConfiguration()
{
  KConfig* config = kapp->config();

  config->setGroup("kjots");
  QFont font( KGlobalSettings::generalFont() );
  font = config->readFontEntry("Font", &font);
  me_text->setFont(font);
  toolBar(0)->applySettings(config, "MainToolbar");
}

void KJotsMain::writeBook()
{
    KJotsBook* b = currentBook();

    if (!b)
        return;

    KURL url;

    do
    {
        url = KFileDialog::getSaveURL();
    } while (!url.isEmpty() && !b->saveToFile(url));
}

void KJotsMain::writePage()
{
//TODO: Merge with writeBook, as saveToFile is virtual anyway
    KJotsSubject* e = dynamic_cast<KJotsSubject*>(subjectList->currentItem());

    if (!e)
        return;

    KURL url;

    do
    {
        url = KFileDialog::getSaveURL();
    } while (!url.isEmpty() && !e->saveToFile(url));
}

void KJotsMain::configureKeys()
{
  KKeyDialog::configure(actionCollection(), this);
}

void KJotsMain::copySelection()
{
//TODO: Strip HTML tags
    KJotsSubject* cur=dynamic_cast<KJotsSubject*>(subjectList->currentItem());

    if (cur && me_text->hasSelectedText())
        cur->setSubject(me_text->selectedText());
}

void KJotsMain::slotPrintPage()
{
//TODO: PORT TO NEW CODEBASE
    /*
    QString docName = current_folder_name;
    KJotsEntry* current = static_cast<KJotsEntry*>(subjectList->currentItem());
    current->setSubject(me_text->selectedText());
    if (current && !current->subject().isNull())
    {
        docName += ": " + current->subject();
    }
    me_text->print(docName);
    */
 }

void KJotsMain::slotQuit()
{
    close();
}

void KJotsMain::insertDate()
{
  me_text->insert(KGlobal::locale()->formatDateTime(QDateTime::currentDateTime(), true) + " ");
}

void KJotsMain::syncTextWithEntry(KJotsSubject* entry)
{
    if (!entry)
        return;

    me_text->setText(entry->body());
    me_text->removeSelection();
    me_text->repaint();
}

void KJotsMain::syncEntryWithText(KJotsSubject* entry)
{
    if (!entry)
        return;

    entry->setBody(me_text->text());
}

void KJotsMain::updateMenu()
{
    KJotsEntryBase* cur=dynamic_cast<KJotsEntryBase*>(subjectList->currentItem());

    if (!cur)
    {
        // no selection
        actionCollection()->action("del_folder")->setEnabled(false);
        actionCollection()->action("del_page")->setEnabled(false);
        actionCollection()->action(KStdAction::stdName(KStdAction::New))->setEnabled(false);
        actionCollection()->action("save_book_to_ascii")->setEnabled(false);
        actionCollection()->action("save_page_to_ascii")->setEnabled(false);
        actionCollection()->action("insert_date")->setEnabled(false);
    }
    else if (cur->isBook())
    {
        // book is selected
        actionCollection()->action("del_folder")->setEnabled(true);
        actionCollection()->action("del_page")->setEnabled(false);
        actionCollection()->action(KStdAction::stdName(KStdAction::New))->setEnabled(true);
        actionCollection()->action("save_book_to_ascii")->setEnabled(true);
        actionCollection()->action("save_page_to_ascii")->setEnabled(false);
        actionCollection()->action("insert_date")->setEnabled(true);
    }
    else
    {
        // subject is selected
        actionCollection()->action("del_folder")->setEnabled(true);
        actionCollection()->action("del_page")->setEnabled(true);
        actionCollection()->action(KStdAction::stdName(KStdAction::New))->setEnabled(true);
        actionCollection()->action("save_book_to_ascii")->setEnabled(true);
        actionCollection()->action("save_page_to_ascii")->setEnabled(true);
        actionCollection()->action("insert_date")->setEnabled(true);
    }
}

void KJotsMain::slotFindSuccessful()
{
    actionCollection()->action(KStdAction::stdName(KStdAction::FindNext))->setEnabled(true);
}

void KJotsMain::showListviewContextenu(KListView*, QListViewItem* i, const QPoint& p)
{
    KActionMenu* am = new KActionMenu(this);

    if (!i)
    {
        // clicked on empty space
        am->insert(actionCollection()->action("new_book"));
    }
    else if (dynamic_cast<KJotsEntryBase*>(i)->isBook())
    {
        // clicked on book
        am->insert(actionCollection()->action("del_folder"));
        am->insert(actionCollection()->action(KStdAction::stdName(KStdAction::New)));
        am->insert(actionCollection()->action("save_book_to_ascii"));
    }
    else
    {
        //clicked on subject
        am->insert(actionCollection()->action("del_page"));
        am->insert(actionCollection()->action("save_page_to_ascii"));
    }

    am->popup(p);
}

bool KJotsMain::queryClose(void)
{
    KJotsSubject* cur = dynamic_cast<KJotsSubject*>(subjectList->currentItem());
    if (cur)
        syncEntryWithText(cur);

    QListViewItem* b = subjectList->firstChild();

    while (b)
    {
        KJotsBook *book = dynamic_cast<KJotsBook*>(b);
        if (book)
        {
            book->closeBook();
        }

        b = b->nextSibling();
    }

    return true;
}

void KJotsMain::slotExpandBook(QListViewItem* item)
{
    void(dynamic_cast<KJotsBook*>(item)->openBook());
}

void KJotsMain::slotCollapseBook(QListViewItem* item)
{
    if (currentEntry)
        syncEntryWithText(dynamic_cast<KJotsSubject*>(currentEntry));

    static_cast<KJotsBook*>(item)->closeBook();
    me_text->clear();
    setCaption(kapp->makeStdCaption(QString::null));
    currentEntry=0;
}

void KJotsMain::jumpToEntry(QListViewItem* entry)
{
    if (currentEntry == entry)
        return;

    if (currentEntry)
        syncEntryWithText(dynamic_cast<KJotsSubject*>(currentEntry));

    if (entry == 0 || static_cast<KJotsEntryBase*>(entry)->isBook() )
    {
        setCaption(kapp->makeStdCaption(QString::null));
        me_text->clear();
        currentEntry = 0;
    }
    else
    {
        syncTextWithEntry(dynamic_cast<KJotsSubject*>(entry));
        me_text->setEnabled(true);
        me_text->setFocus();

        // TODO: this will have to be fixed if/when we allow arbitrary depth pages
        QString caption = dynamic_cast<KJotsEntryBase*>(entry->parent())->subject() +
                          " / " +
                          dynamic_cast<KJotsEntryBase*>(entry)->subject();
        setCaption(kapp->makeStdCaption(caption));
        currentEntry = static_cast<KJotsEntryBase*>(entry);
    }

    updateMenu();
}

void KJotsMain::slotItemRenamed(QListViewItem* item,const QString& str, int  /*col*/)
{
    if (!dynamic_cast<KJotsEntryBase*>(item)->isBook())
        return;

    KJotsBook* b=dynamic_cast<KJotsBook*>(item);

    if (b->beforeRename() == str)
        return;

    QDir dir=QDir(locateLocal("appdata",""));
    QString file_name = dir.absPath();
    file_name += '/' + QFile::encodeName(str);

    if (QFile::exists(file_name))
    {
        KMessageBox::sorry(this, i18n("A book by that name already exists."), i18n("Rename Book"));
        b->setSubject(b->beforeRename());
        return;
    }

    dir.rename(b->beforeRename(), str);
}

void KJotsMain::slotItemMoved(QListViewItem *item, QListViewItem * afterFirst, QListViewItem * /* afterNow */)
{
    KJotsEntryBase *p_item = dynamic_cast<KJotsEntryBase*>(item);
    KJotsEntryBase *p_item_parent = dynamic_cast<KJotsEntryBase*>(item->parent());
    if (p_item_parent)
    {
        if (p_item->isBook() && p_item_parent->isBook())
        {
            // we don't accept books inside books right now
            // move the book outside
            KMessageBox::sorry(this,i18n("You can't have a book inside another book."));
            subjectList->moveItem(item, 0, afterFirst);
        }
    }
    else if (!p_item->isBook())
    {
        KJotsSubject *p_aux = dynamic_cast<KJotsSubject*>(item);

        if (!p_aux)
        {
            return;
        }

        // we don't accept subjects in the root
        // create a new book
        if (KMessageBox::questionYesNo(this,
                                       i18n("All subjects must be inside a book. "
                                            "Would you like to create a new book to put the subject in, "
                                            "or would you prefer to not move the subject at all?"),
                                       QString::null,
                                       i18n("Create New Book"), i18n("Don't Move The Subject")) ==
            KMessageBox::No ||
            !createFolder())
        {
            subjectList->moveItem(p_aux, currentBook(), afterFirst);
            return;
        }

        delete currentBook()->firstChild();
        currentBook()->insertItem(p_aux);
        me_text->setText(p_aux->body());
        currentEntry = p_item;
    }
}

KJotsBook* KJotsMain::currentBook(void)
{
    KJotsEntryBase* e = dynamic_cast<KJotsEntryBase*>(subjectList->currentItem());

    while (e)
    {
        if (e->isBook())
        {
            return dynamic_cast<KJotsBook*>(e);
        }
        e = dynamic_cast<KJotsEntryBase*>(e->parent());
    }

    return 0;
}

#include "KJotsMain.moc"
/* ex: set tabstop=4 softtabstop=4 shiftwidth=4 expandtab: */
