//
//  kjots
//
//  Copyright (C) 1997 Christoph Neerfeld
//  Copyright (C) 2002, 2003 Aaron J. Seigo
//  Copyright (C) 2003 Stanislav Kljuhhin
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#include <iostream>
#include <qdir.h>
#include <qfile.h>

#include <klocale.h>
#include <kstringhandler.h>
#include <kicontheme.h>
#include <kiconloader.h>
#include <kstandarddirs.h>
#include <kurl.h>
#include <kmessagebox.h>

#include "kjotsentry.h"

#include <iostream>
using namespace std;

extern KIconLoader* global_pix_loader;

//
// KJotsEntryBase
//

KJotsEntryBase::KJotsEntryBase(KListView* parent,QListViewItem* after=0)
	:KListViewItem(parent,after)
{

}

KJotsEntryBase::KJotsEntryBase(KListViewItem* parent,QListViewItem* after=0)
	:KListViewItem(parent,after)
{

}

void KJotsEntryBase::setSubject(const QString& subj)
{
    setText(0,subj);
}

void KJotsEntryBase::setText(int column,const QString& text)
{
    if (column==0&&(text.isNull()||text.length()<1))
        KListViewItem::setText(0, i18n("Untitled"));
    else
        KListViewItem::setText(column, text);
}

//
// KJotsBook
//

KJotsBook::KJotsBook(KListView* parent, QString subj, QListViewItem* after)
    : KJotsEntryBase(parent, after),
    m_open(true),
    m_deleteFlag(false)
{
    setSubject(subj);
    setExpandable(true);
    setPixmap(0,global_pix_loader->loadIcon(QString("contents"),KIcon::Small));
}

KJotsBook::~KJotsBook()
{
    if (m_deleteFlag)
        annihilateBook();
    else
        closeBook();
}

bool KJotsBook::openBook(void)
{
    listView()->setUpdatesEnabled(false);

    QDir dir=QDir(locateLocal("appdata",""));
    QString file_name=dir.absPath();
    file_name+='/';
    file_name+=QFile::encodeName(subject());

    QFile folder(file_name);
  
    if( !folder.open(IO_ReadWrite) )
    {
        listView()->setUpdatesEnabled(true);
        return false;
    }

    QTextStream st(static_cast<QIODevice*>(&folder));
    QString buf = st.readLine();
  
    if(buf.left(9)!="\\NewEntry")
    {
        listView()->setUpdatesEnabled(true);
        return false;
    }

    QString subj=buf.mid(10,buf.length());
    QString body;
    QString title=subj.isNull()?QString::null:QString(subj);
    
    KJotsSubject *entry=new KJotsSubject(this,title);

    while(!st.eof())
    {
        buf=st.readLine();
        
        if(buf.left(9)=="\\NewEntry")
        {
            entry->setBody(body);
            body=QString::null;
            subj=buf.mid(10,buf.length());
            title=subj.isNull()?QString(""):subj;
            entry=new KJotsSubject(this,title,entry);
            buf=st.readLine();
        }

        int pos=0;
    
        while((pos=buf.find('\\',pos))!=-1)
            if(buf[++pos]=='\\')
                buf.remove(pos, 1 );

        body.append(buf + "\n");
    }

    entry->setBody(body);
    
    folder.close();

    listView()->setUpdatesEnabled(true);

    m_open=true;
    return true;
}

void KJotsBook::closeBook(void)
{
    if (!m_open)
        return;

    listView()->setUpdatesEnabled(false);

    QDir dir = QDir(locateLocal("appdata",""));
    QString file_name = dir.absPath();
    file_name +=  '/' + subject();

    QFile folder(file_name);
    if(!firstChild() || !folder.open(IO_WriteOnly | IO_Truncate))
    {
        listView()->setUpdatesEnabled(true);
        m_open = false;

        return;
    }

    KJotsSubject* entry = dynamic_cast<KJotsSubject*>(firstChild());

    QTextStream st((QIODevice *) &folder);

    while (entry)
    {
        st << "\\NewEntry " << entry->subject() << "\n";
        QString buf = entry->body();
        st << buf.replace('\\', "\\\\");
        if( buf.right(1) != "\n" )
        {
            st << '\n';
        }

        KJotsSubject* newEntry = dynamic_cast<KJotsSubject*>(entry->nextSibling());
        delete entry;
        entry = newEntry;
    }

    folder.close();

    listView()->setUpdatesEnabled(true);

    m_open = false;
}

void KJotsBook::annihilateBook(void)
{
    QDir dir = QDir(locateLocal("appdata",""));
    dir.remove(subject());
}

void KJotsBook::startRename(int col)
{
    m_beforeRename = text(0);
    KListViewItem::startRename(col);
}

bool KJotsBook::saveToFile(KURL url)
{
    if ( url.isEmpty() )
        return false;

    if ( !url.isLocalFile() )
    {
        KMessageBox::sorry( 0L, i18n( "Only local files are currently supported." ) );
        return false;
    }

    QString name = url.path();

    QFileInfo f_info(name);
    if( f_info.exists() )
    {
        QString msg = i18n("File already exists.\n"
                "Do you want to overwrite it?");
        int result = KMessageBox::warningYesNo(listView()->topLevelWidget(), msg );
        if( result != KMessageBox::Yes )
            return false;
    }

    QFile ascii_file(name);
    if( !ascii_file.open(IO_WriteOnly | IO_Truncate) )
        return false;
    QTextStream st((QIODevice *) &ascii_file);

    KJotsSubject* entry=dynamic_cast<KJotsSubject*>(this->firstChild());

    while (entry)
    {
        entry->write(st);
        entry=dynamic_cast<KJotsSubject*>(entry->nextSibling());
    }

    ascii_file.close();

    return true;
}

//
// KJotsSubject
//

KJotsSubject::KJotsSubject(KJotsBook* parent, QString subj, QListViewItem* after)
    : KJotsEntryBase(parent,after)
{
    setSubject(subj);
    setPixmap(0,global_pix_loader->loadIcon(QString("edit"),KIcon::Small));
}

QString KJotsSubject::body(void)
{
    return m_text;
}

void KJotsSubject::setBody(const QString& text)
{
    m_text = text;
}

void KJotsSubject::write(QTextStream &st)
{
    QString line, buf;
    line.fill('#', subject().length() + 2 );
    st << line << '\n';
    st << "# ";
    st << subject() << '\n';
    st << line << '\n';
    buf = body();
    st << buf;
    if( buf.right(1) != "\n" )
        st << '\n';
    st << '\n';
}

bool KJotsSubject::saveToFile(KURL url)
{
    if(url.isEmpty())
        return false;

    if(!url.isLocalFile())
    {
        KMessageBox::sorry(0L,i18n( "Only local files are currently supported."));
        return false;
    }

    QString name=url.path();

    QFileInfo f_info(name);
    if(f_info.exists())
    {
        QString msg=i18n("File already exists.\n"
                "Do you want to overwrite it?");
        int result=KMessageBox::warningYesNo(listView()->topLevelWidget(),msg);
        if(result!=KMessageBox::Yes)
            return false;
    }

    QFile ascii_file(name);
    if( !ascii_file.open(IO_WriteOnly | IO_Truncate) )
        return false;
    QTextStream st((QIODevice *) &ascii_file);
    write(st);
    ascii_file.close();

    return true;
}

/* ex: set tabstop=4 softtabstop=4 shiftwidth=4 expandtab: */
